//  ComputationalAssembler.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

import swarm.collections.ListImpl;
import swarm.collections.ListIndex;


/**
 * The ComputationalAssembler class instances make 
 * computational processes; we have a computationa assembler instance for
 * each unit
 *
 * @author Pietro Terna 
 */
public class ComputationalAssembler extends ComputationalAssemblerBasic{

    float i0, i1, o0, o1; // variable used to rotate the direction of the
                          // Langton Ants

    /**
     * the constructor for ComputationalAssembler
     */

    public ComputationalAssembler (Zone aZone, OrderDistiller od, ESFrameModelSwarm mo)
    {
	// Call the constructor for the parent class.
	super(aZone, od, mo);
    }

    /** drop the computational assembler */
    public void drop(){

	InformationRepository.addToHibernatedComputationalAssemblerList(this);
    }

    // methods for Langton Ants



    /** moving an ant raqndomly near to its position
     *  this computational code acts via the c1103 code to create, with the probability set in 
     *  position 0,0 of the first received matrix a copy  of the unit the order is in, near to it
     *  in position 0,1 of the second received matrix we have the number of copied units
     *
     *  then it drops the unit the order is in via the c1102 code, with the probability set in 
     *  position 1,0 of the first received matrix (c1101 see it with rd=1 as 0+rd,0+cd)
     *  in position 1,1 of the second received matrix we have the number of dropped units
     *
     *  finally the unit is copied (for dispaly purposes) in the stratum reported 
     *  in position 2,1 of the firts received matrix, with the probability set
     *  in position 2,0 of the same matrix, modifying the production phase of the 
     *  the unit in the other stratum with the addedndum reported 
     *  in position 2,2 of the same matrix (the difference in production phase avoids that
     *  the unit in the other stratum can receive the oders destinated to the units of the 
     *  orginal stratum
     *
     *  done is set to true in c1103, c1102, c1111
     */
    public void c1301()
    {
	Order theCurrentOrder;
	Unit theUnitTheOrderIsIn, newUnit, deletedUnit;
	MemoryMatrix mU, mUNew;
	int layer=0;

	if(pendingComputationalSpecificationSet.
	   getNumberOfMemoryMatrixesToBeUsed()!=2)
	    {
		System.out.println("Code -1301 requires 2 matrixes; " + 
				   pendingComputationalSpecificationSet.
				   getNumberOfMemoryMatrixesToBeUsed() + 
				   " found in order # " +
				   pendingComputationalSpecificationSet.
				   getOrderNumber());
		MyExit.exit(1);
	    }

	// our memory matrix
	theCurrentOrder=pendingComputationalSpecificationSet.getOrder();
	theUnitTheOrderIsIn=theCurrentOrder.getTheUnitTheOrderIsIn();
	mU=theUnitTheOrderIsIn.getMemoryMatrix();

	// checking our position in the other stratum and deleting an exisiting unit
	checkMatrixNumber=false;
	rd=3;
	deletedUnit=c1130();
	rd=0;
	checkMatrixNumber=true;

	// setting the initial direction (North, being the maps built with a reversed y axis)
	if(mU.getValue(layer,0,0)==0 && mU.getValue(layer,0,1)==0)
	    {
		mU.setValue(layer,0,0,0.0);
		mU.setValue(layer,0,1,-1.0);
	    }
	else if (deletedUnit==null)
	    {
		i0=mU.getValue(layer,0,0);
		i1=mU.getValue(layer,0,1);
		rotateRight();
		mU.setValue(layer,0,0,o0);
		mU.setValue(layer,0,1,o1);
	    }
	else
	    {
		i0=mU.getValue(layer,0,0);
		i1=mU.getValue(layer,0,1);
		rotateLeft();
		mU.setValue(layer,0,0,o0);
		mU.setValue(layer,0,1,o1);
	    }

	// creating a copy of the unit near the previous 
	// one with a stated direction and the standard placement correction if the 
	// destinarion place is not free
	newUnit=c1105();

	// copying memory matrix
	if (newUnit != null)
	    {
		mUNew=newUnit.getMemoryMatrix();

		for (int i=0; i< mU.getNumberOfRows();i++)
		    for (int j=0; j< mU.getNumberOfCols();j++)
			mUNew.setValue(layer,i,j,mU.getValue(layer,i,j));
	    }


	// copy the old unit to another stratum (the unit the order is in)
	if(deletedUnit==null)
	    {
		mU.setValue(layer,1,0,1.0); // 1.0 as a generic positive number
		                            // it will be checked by c1111 to decide if
		                            // to copy the unit
		checkMatrixNumber=false;
		urd=1; // check pos 1,0 in unit memory matrix
		rd=2;
		c1111();
		checkMatrixNumber=true;
		urd=0;
		rd=0;
	    }
		
	// deleting the unit in previous position (the unit the order is in)
	rd=1;
	c1102();
	rd=0;

    } // end c1301

    /**
     *  internal method to rotate the direction vector
     */
    public void rotateRight()
    {
	// the rotation il reversed, being reversed also reversed the y axis in the maps
	o0=i0*0+i1*(-1);
	o1=i0*(1)+i1*0;

    } // end rotateRight

    public void rotateLeft()
    {
	
	o0=i0*0+i1*(1);
	o1=i0*(-1)+i1*0;

    } // end rotateLeft


}
