//  ComputationalAssembler.java

import swarm.Globals;
import swarm.defobj.Zone;
import swarm.objectbase.SwarmObjectImpl;

import swarm.collections.ListImpl;
import swarm.collections.ListIndex;


/**
 * The ComputationalAssembler class instances make 
 * computational processes; we have a computationa assembler instance for
 * each unit
 *
 * @author Pietro Terna 
 */
public class ComputationalAssembler extends ComputationalAssemblerBasic{

    /**
     * the constructor for ComputationalAssembler
     */

    public ComputationalAssembler (Zone aZone, OrderDistiller od, ESFrameModelSwarm mo)
    {
	// Call the constructor for the parent class.
	super(aZone, od, mo);

    }

    /** drop the computational assembler */
    public void drop(){

	InformationRepository.addToHibernatedComputationalAssemblerList(this);
    }

    /** computational operations with code -12xx (to be kept in 
     *  ComputationalAssembler and not to be transferred in 
     *  ComputationalAssemblerBasic) <br><br>
     *
     */


    /** unit display on another stratum
     *  <br><br>
     *  the computational step 1250 (a code for the workers_skills_firms 
     *  application) acts via 1110 code
     *
     *  when received by one unit in a stratum, acts vs. all the units of the same 
     *  type (i.e., with the same production phase) in all strata (excluding those 
     *  of the destination stratum), copying them on the destination stratum
     *
     *  finally it changes the status to done; the change is also done in 1110, but in 
     *  case non similar units were found, ww have to repeat it there
     */
    public void c1250(){
	int i, im, thePhase, unitCount, layer=0;
	Unit theUnitTheOrderIsIn;
	Order theCurrentOrder;
	ListImpl aList;
	MemoryMatrix mU;


	if(pendingComputationalSpecificationSet.
	   getNumberOfMemoryMatrixesToBeUsed()!=1 && checkMatrixNumber)
	    {
		System.out.println("Code -1250 requires one matrix; " + 
				   pendingComputationalSpecificationSet.
				   getNumberOfMemoryMatrixesToBeUsed() + 
				   " found in order # " +
				   pendingComputationalSpecificationSet.
				   getOrderNumber());
		MyExit.exit(1);
	    }

	theCurrentOrder=pendingComputationalSpecificationSet.getOrder();
	theUnitTheOrderIsIn=theCurrentOrder.getTheUnitTheOrderIsIn();
	thePhase=theUnitTheOrderIsIn.getProductionPhase();
	
	ucd=1; // the actual position of the control cell for c1110 in the memory 
	       // matrix of the unit is displaced of 1 column
	for (im=0;im<InformationRepository.getStratumNumber();im++)
	    {
		aList=(ListImpl) InformationRepository.getUnitListFromModelSequence(im);
		unitCount=aList.getCount();
		for (i=0; i<unitCount;i++)
		    {
			// whichUnit is substituting, in c1110, theUnitTheOrderIsIn
			// because here we apply c1110 to a list
			whichUnit=(Unit) aList.atOffset(i);
			// is the unit showing the same production phase of the
			// receiving unit?
			if(thePhase==whichUnit.getProductionPhase())
			    {
				mU=whichUnit.getMemoryMatrix();
				mU.setValue(layer,0,1,1.0);
				c1110();
			    }
		    }
	    }

	ucd=0;
	whichUnit=null;
	done=true; // in case no c1110 step was executed

    } // end c1250


    /** unit display on another stratum, case of units with phase 1
     *  <br><br>
     *  the computational step 1251 (a code for the workers_skills_firms 
     *  application) acts via 1250 code, setting to 1 the row used 
     *  in matrix 0
     */
    public void c1251(){
	if(pendingComputationalSpecificationSet.
	   getNumberOfMemoryMatrixesToBeUsed()!=1 && checkMatrixNumber)
	    {
		System.out.println("Code -1251 requires one matrix; " + 
				   pendingComputationalSpecificationSet.
				   getNumberOfMemoryMatrixesToBeUsed() + 
				   " found in order # " +
				   pendingComputationalSpecificationSet.
				   getOrderNumber());
		MyExit.exit(1);
	    }
	rd=1;
	c1250();
	rd=0;
    } // end c1251

    /** unit display on another stratum, case of units with phase 2
     *  <br><br>
     *  the computational step 1251 (a code for the workers_skills_firms 
     *  application) acts via 1250 code, setting to 2 the row used 
     *  in matrix 0
     */
    public void c1252(){
	if(pendingComputationalSpecificationSet.
	   getNumberOfMemoryMatrixesToBeUsed()!=1 && checkMatrixNumber)
	    {
		System.out.println("Code -1252 requires one matrix; " + 
				   pendingComputationalSpecificationSet.
				   getNumberOfMemoryMatrixesToBeUsed() + 
				   " found in order # " +
				   pendingComputationalSpecificationSet.
				   getOrderNumber());
		MyExit.exit(1);
	    }
	rd=2;
	c1250();
	rd=0;
    } // end c1252

    /** unit display on another stratum, case of units with phase 3
     *  <br><br>
     *  the computational step 1251 (a code for the workers_skills_firms 
     *  application) acts via 1250 code, setting to 3 the row used 
     *  in matrix 0
     */
    public void c1253(){
	if(pendingComputationalSpecificationSet.
	   getNumberOfMemoryMatrixesToBeUsed()!=1 && checkMatrixNumber)
	    {
		System.out.println("Code -1253 requires one matrix; " + 
				   pendingComputationalSpecificationSet.
				   getNumberOfMemoryMatrixesToBeUsed() + 
				   " found in order # " +
				   pendingComputationalSpecificationSet.
				   getOrderNumber());
		MyExit.exit(1);
	    }
	rd=3;
	c1250();
	rd=0;
    } // end c1253

}
